﻿namespace TabsStudioSingleRow
{
    public class HiddenTabs : System.Windows.Controls.Button
    {
        public HiddenTabs()
        {
            Click += OnClick;
        }

        private void OnClick(object sender, System.Windows.RoutedEventArgs e)
        {
            if (TabsPanel == null)
                return;

            System.Windows.Controls.ContextMenu menu = new System.Windows.Controls.ContextMenu();
            foreach (TabsStudioExt.Tabs tabs in SingleRow.GetEngine().GetTabs())
            {
                if (tabs.TabPanel2 == TabsPanel)
                {
                    System.Collections.Generic.List<System.Windows.Controls.MenuItem> menuItems =
                        new System.Collections.Generic.List<System.Windows.Controls.MenuItem>();
                    foreach (System.Windows.Controls.TabItem i in TabsPanel.Children)
                    {
                        if (Properties.GetIsTabHidden(i))
                        {
                            System.Windows.Controls.MenuItem item = CreateMenuItem(i, tabs);
                            if (item != null)
                                menuItems.Add(item);
                        }
                    }
                    menuItems.Sort(
                        delegate(System.Windows.Controls.MenuItem lhs, System.Windows.Controls.MenuItem rhs)
                            { 
                                return (lhs.Header as string).CompareTo(rhs.Header as string); 
                            }
                        );
                    foreach (System.Windows.Controls.MenuItem i in menuItems)
                        menu.Items.Add(i);
                    break;
                }
            }
            menu.IsOpen = true;
        }

        private System.Windows.Controls.MenuItem CreateMenuItem(
            System.Windows.Controls.TabItem i, TabsStudioExt.Tabs tabs)
        {
            TabsStudioExt.Tab tab = FindTabByItem(i, tabs);
            if (tab == null)
                return null;

            System.Windows.Controls.MenuItem result = new System.Windows.Controls.MenuItem();
            string header = tab.TitleManager.Title;
            if (tab.TabExtensions.Count > 1)
            {
                foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
                {
                    header += " " + ext.TitleManager.Title;
                }
            }
            result.Header = GuardMenuUnderscores(header);

            foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
            {
                if (ext.IsExtensionActive)
                {
                    result.Command = new ActivateHiddenTabCommand(ext.Window);
                    break;
                }
            }

            return result;
        }

        private TabsStudioExt.Tab FindTabByItem(System.Windows.Controls.TabItem tabItem, TabsStudioExt.Tabs tabs)
        {
            foreach (TabsStudioExt.Tab tab in tabs.TabList)
            {
                if (tab.TabItem == tabItem)
                    return tab;
            }
            return null;
        }

        private string GuardMenuUnderscores(string s)
        {
	        return s.Replace("_", "__");
        }

        static private void OnTabsPanelChanged(System.Windows.DependencyObject hiddenTabs, 
            System.Windows.DependencyPropertyChangedEventArgs e)
        {
            HiddenTabs thisObject = hiddenTabs as HiddenTabs;
            if (thisObject.TabsPanel == null)
                return;

            Properties.BindProperty(thisObject.TabsPanel, Properties.IsHiddenTabsProperty, 
                thisObject, Properties.IsHiddenTabsProperty);
        }

        public System.Windows.Controls.Panel TabsPanel
        {
            set { SetValue(TabsPanelProperty, value); }
            get { return (System.Windows.Controls.Panel)GetValue(TabsPanelProperty); }
        }

        public static readonly System.Windows.DependencyProperty TabsPanelProperty =
            System.Windows.DependencyProperty.Register("TabsPanel", typeof(System.Windows.Controls.Panel),
                typeof(HiddenTabs), new System.Windows.FrameworkPropertyMetadata(OnTabsPanelChanged));
    }
}
