﻿namespace TabsStudioSingleRow
{
    public class SingleRow : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IStyler, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            storage = new OptionsStorage();
            options = storage.Load();
            if (dte.Version.Equals("10.0"))
                stateSaver = new HiddenStateSaver();
            monitors = new System.Collections.Generic.List<TabsMonitor>();
         
            engine.TabsCreated += OnTabsCreated;
            engine.TabsDestroyed += OnTabsDestroyed;
        }

        public void OnDisconnection()
        {
            engine.TabsCreated -= OnTabsCreated;
            engine.TabsDestroyed -= OnTabsDestroyed;

            foreach (TabsMonitor i in monitors)
                i.Close();

            if (stateSaver != null)
                stateSaver.Close();

            engine = null;
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioSingleRow=\"clr-namespace:TabsStudioSingleRow;assembly=SingleRow\"";
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
            {
                storage.Save(options);
                foreach (TabsMonitor i in monitors)
                    i.UpdateOptions();
            }
        }

        private void OnTabsCreated(object sender, TabsStudioExt.TabsEventArgs e)
        {
            monitors.Add(new TabsMonitor(options, e.Tabs, stateSaver));
        }

        private void OnTabsDestroyed(object sender, TabsStudioExt.TabsEventArgs e)
        {
            foreach (TabsMonitor i in monitors)
            {
                if (i.GetPanel() == e.Tabs.TabPanel2)
                {
                    i.Close();
                    monitors.Remove(i);
                    break;
                }
            }
        }

        static public TabsStudioExt.ITabsStudioEngine GetEngine()
        {
            return engine;
        }

        static private TabsStudioExt.ITabsStudioEngine engine;
        private OptionsStorage storage;
        private Options options;
        private HiddenStateSaver stateSaver;
        private System.Collections.Generic.List<TabsMonitor> monitors;
    }
}
