﻿namespace TabsStudioSingleRow
{
    class TabsMonitor
    {
        public TabsMonitor(Options options_, TabsStudioExt.Tabs tabs, HiddenStateSaver stateSaver_)
        {
            options = options_;
            panel = tabs.TabPanel2; 
            stateSaver = stateSaver_;
            tabsUsageOrder = new System.Collections.Generic.List<System.Windows.Controls.TabItem>();

            heightDescriptor = System.ComponentModel.DependencyPropertyDescriptor.FromProperty(
                    System.Windows.FrameworkElement.ActualHeightProperty, typeof(System.Windows.FrameworkElement));
            heightDescriptor.AddValueChanged(panel, OnTabsHeightChanged);

            SingleRow.GetEngine().TabCreated += OnTabCreated;
            SingleRow.GetEngine().TabDestroyed += OnTabDestroyed;

            UpdateHiddenTabs();
        }

        public void Close()
        {
            SingleRow.GetEngine().TabCreated -= OnTabCreated;
            SingleRow.GetEngine().TabDestroyed -= OnTabDestroyed;

            heightDescriptor.RemoveValueChanged(panel, OnTabsHeightChanged);
        }

        public System.Windows.Controls.Panel GetPanel()
        {
            return panel;
        }

        public void UpdateOptions()
        {
            UpdateTabs();
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (e.Tabs.TabPanel2 != panel)
                return;

            if (isTabSelectedDescriptor == null)
                isTabSelectedDescriptor = System.ComponentModel.DependencyPropertyDescriptor.FromName(
                    "IsTabSelected", e.Tab.TabItem.GetType(), e.Tab.TabItem.GetType());
            isTabSelectedDescriptor.AddValueChanged(e.Tab.TabItem, OnTabSelectedChanged);

            Properties.SetIsTabHidden(e.Tab.TabItem, false);

            if (stateSaver != null && stateSaver.IsHidden(e.Tab.TabExtensions[0].Window))
                tabsUsageOrder.Insert(0, e.Tab.TabItem);
            else
                tabsUsageOrder.Add(e.Tab.TabItem);

            foreach (System.Windows.Controls.TabItem i in tabsUsageOrder)
            {
                if ((bool)isTabSelectedDescriptor.GetValue(i))
                {
                    tabsUsageOrder.Remove(i);
                    tabsUsageOrder.Add(i);
                    break;
                }
            }
        }

        private void OnTabDestroyed(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (e.Tabs.TabPanel2 != panel)
                return;

            tabsUsageOrder.Remove(e.Tab.TabItem);
            isTabSelectedDescriptor.RemoveValueChanged(e.Tab.TabItem, OnTabSelectedChanged);
            UpdateHiddenTabs();
        }

        private void OnTabSelectedChanged(object sender, System.EventArgs e)
        {
            if ((bool)isTabSelectedDescriptor.GetValue(sender))
            {
                System.Windows.Controls.TabItem tabItem = sender as System.Windows.Controls.TabItem;
                Properties.SetIsTabHidden(tabItem, false);

                tabsUsageOrder.Remove(tabItem);
                tabsUsageOrder.Add(tabItem);

                UpdateHiddenTabs();
            }
        }

        private void OnTabsHeightChanged(object sender, System.EventArgs e)
        {
            UpdateTabs();
        }

        private void UpdateTabs()
        {
            while (GetNumberOfRows() > options.maxRows && HideNextTab())
                panel.UpdateLayout();
        }

        private uint GetNumberOfRows()
        {
            if (panel.Children.Count == 0)
                return 0;

            uint result = 0;
            double currentTop = 0;
            foreach (System.Windows.Controls.TabItem i in panel.Children)
            {
                if (!Properties.GetIsTabHidden(i))
                {
                    double top = i.TranslatePoint(new System.Windows.Point(0, 0), panel).Y - i.Margin.Top;
                    if (result == 0 || top != currentTop)
                    {
                        ++result;
                        currentTop = top;
                    }
                }
            }
            return result;
        }

        private bool HideNextTab()
        {
            foreach (System.Windows.Controls.TabItem i in tabsUsageOrder)
            {
                if (!Properties.GetIsTabHidden(i))
                {
                    Properties.SetIsTabHidden(i, true);
                    UpdateHiddenTabs();
                    return true;
                }
            }
            return false;
        }

        private void UpdateHiddenTabs()
        {
            bool hidden = false;
            foreach (System.Windows.Controls.TabItem i in panel.Children)
            {
                if (Properties.GetIsTabHidden(i))
                {
                    hidden = true;
                    break;
                }
            }
            Properties.SetIsHiddenTabs(panel, hidden);
        }

        private Options options;
        private System.Windows.Controls.Panel panel;
        private HiddenStateSaver stateSaver;
        private System.Collections.Generic.List<System.Windows.Controls.TabItem> tabsUsageOrder;
        System.ComponentModel.DependencyPropertyDescriptor heightDescriptor;
        private System.ComponentModel.DependencyPropertyDescriptor isTabSelectedDescriptor;
    }
}
